@@ -22,6 +22,9 @@ impl Error {
     }
 
     pub fn new(span: Span, kind: ErrorKind) -> Self {
+        if cfg!(debug_assertions) && span.is_dummy() {
+            panic!("parser should not create an error with dummy span")
+        }
         Error {
             inner: Box::new((span, kind)),
         }
